/**************/
/* scalin4.c  */
/****************************************************************************
   PROGRAM: scalin4.c
   PURPOSE: Vaihtaa matkoja
            Huolehditaan scaling-dialogin ksittelyst.
            Dialogien lokaalin muisti ksittely => samasta dialogista voi
            olla useita esiintymi.

            Vaatii ikkunakahvakohtaisen muistialueen lukituksen ennen
            ikkunalle kuuluvan muistilohkon ksittely.

            Lukitus hoidetaan tss automaattisella kutsulla ennen ja
            jlkeen EVENT-kutsua.

            LOCK-makro huolehtii siit, ett tapahtumataulukkoon tulee
            tarvittavat ennen kutsua kytettv funktio ja jlkeen
            kytettv funktio.

****************************************************************************/
#include <windowsx.h>
#include "portable.h"
#include "mdialog.h"
#include "showdoub.h"
#include "tabhand.h"
#include "vaihtaja.h"

/***************************************************************************/
typedef struct {
  char   *unit;
  double mm;
} unit_type;

static unit_type units[] = {
  { "mm",                1.0     },
  { "cm",               10.0     },
  { "dm",              100.0     },
  { "m",              1000.0     },
  { "km",          1000000.0     },
  { "inch",             25.4     },
  { "ft",              304.8     },
  { "yard",            914.4     },
  { "mile",        1609300.0     },
  { "NM",          1853200.0     },
  { "tuuma(s)",         24.742   },
  { "kortteli",        148.45    },
  { "jalka(s)",        296.9     },
  { "kyynr",         593.8     },
  { "syli",           1781.4     },
  { "virsta",      1068800.0     },
  { "peninkulma", 10688000.0     },
  { NULL,                0.0     }
};

#define MM 0
#define KM 4

#define DO_SHOW_OUT 1991
#define DO_SHOW_MAP 1992

#define H(id) GetDlgItem(msg->hWnd,id)
#define SEND_COMMAND(id) SEND_WM_COMMAND(msg->hWnd,id,GetDlgItem(msg->hWnd,id),0)

/***************************************************************************/
/* Lokaali tietorakenne:                                                   */
/***************************************************************************/
typedef struct {
  int    map_unit;
  double map_dist;
  int    out_unit;
  double out_dist;
  double scale;
  int    change_other;
} tLocalStruct, *pLocalStruct;

static int InitScaling(pLocalStruct lp)
/* Lokaalin muistialueen alustus                                           */
{
  lp->map_unit     = MM;          /* mm */
  lp->map_dist     = 10.0;
  lp->out_unit     = KM;          /* km */
  lp->out_dist     = 2.0;
  lp->scale        = 200000.0;
  lp->change_other = 1;
  return 0;
}

/* Luodaan ennen ja jlkeen funktiot ja niiden taulukko:                   */
DO_EVENT_LOCAL_LOCK(pLocalStruct,InitScaling,EventLock,EventUnLock,EventBA);
// #define LOCK 0,0,0,0,{EventLock,EventUnLock}
#define LOCK 0,0,0,0,&EventBA

/* Lyhenne lokaaliin tietorakenteeseen viittaamiseksi.                     */
#define LP ((pLocalStruct)(msg->LocalInfo))
/***************************************************************************/


/***************************************************************************/
static EVENT WM_initdialog(tMSGParam *msg)                         /* # LOCK #*/
{
  int i;
  HWND hCBMapU = GetDlgItem(msg->hWnd,ID_MAP_UNIT);
  HWND hCBOutU = GetDlgItem(msg->hWnd,ID_OUT_UNIT);
  for (i=0; units[i].unit; i++) {  /* Tytetn combo-listat */
    (void)ComboBox_InsertString(hCBMapU,i,units[i].unit);
    (void)ComboBox_InsertString(hCBOutU,i,units[i].unit);
  }

  (void)ComboBox_SetCurSel(hCBMapU,LP->map_unit);              /* oletus yks  */
  (void)ComboBox_SetCurSel(hCBOutU,LP->out_unit);              /* oletus yks  */
  (void)Button_SetCheck(H(IDR_OTHER),LP->change_other=1);      /* oletus muut */

  ShowDouble(msg->hWnd,ID_SCALE,LP->scale);
  ShowDouble(msg->hWnd,ID_MAP_DIST,LP->map_dist);
  return ShowDouble(msg->hWnd,ID_OUT_DIST,LP->out_dist);
}

static EVENT WM_command_DO_SHOW_OUT(tMSGParam *msg)                /* # LOCK #*/
{
  LP->out_dist = LP->map_dist*units[LP->map_unit].mm*LP->scale/units[LP->out_unit].mm;
  return ShowDouble(msg->hWnd,ID_OUT_DIST,LP->out_dist);
}

static EVENT WM_command_DO_SHOW_MAP(tMSGParam *msg)                /* # LOCK #*/
{
  LP->map_dist = LP->out_dist/(units[LP->map_unit].mm*LP->scale/units[LP->out_unit].mm);
  return ShowDouble(msg->hWnd,ID_MAP_DIST,LP->map_dist);
}

static EVENT WM_command_ID_SCALE__EN_CHANGE(tMSGParam *msg)        /* # LOCK #*/
{
  LP->scale = GetDouble(msg->hWnd,msg->wParam,1);
  if ( LP->scale <= 0 ) LP->scale = 1;
  return SEND_COMMAND(DO_SHOW_OUT);
}

static EVENT WM_command_ID_MAP_DIST__EN_CHANGE(tMSGParam *msg)     /* # LOCK #*/
{
  LP->map_dist = GetDouble(msg->hWnd,msg->wParam,0);
  return SEND_COMMAND(DO_SHOW_OUT);
}

static EVENT WM_command_ID_OUT_DIST__EN_CHANGE(tMSGParam *msg)     /* # LOCK #*/
{
  LP->out_dist = GetDouble(msg->hWnd,msg->wParam,0);
  return SEND_COMMAND(DO_SHOW_MAP);
}

static EVENT WM_command_ID_MAP_UNIT__CBN_SELCHANGE(tMSGParam *msg) /* # LOCK #*/
{
  LP->map_unit = (int)SendDlgItemMessage(msg->hWnd,ID_MAP_UNIT,CB_GETCURSEL,0,0);
  return SEND_COMMAND(LP->change_other ? DO_SHOW_OUT : DO_SHOW_MAP);
}

static EVENT WM_command_ID_OUT_UNIT__CBN_SELCHANGE(tMSGParam *msg) /* # LOCK #*/
{
  LP->out_unit = (int)SendDlgItemMessage(msg->hWnd,ID_OUT_UNIT,CB_GETCURSEL,0,0);
  return SEND_COMMAND(LP->change_other ? DO_SHOW_MAP : DO_SHOW_OUT);
}

static EVENT WM_command_IDR_THIS(tMSGParam *msg)                   /* # LOCK #*/
{
  return LP->change_other = 0;
}

static EVENT WM_command_IDR_OTHER(tMSGParam *msg)                  /* # LOCK #*/
{
  return LP->change_other = 1;
}

static EVENT WM_command_IDOK(tMSGParam *msg)
{
  SetFocus(H(ID_MAP_DIST)); Edit_SetSel(H(ID_MAP_DIST),0,-1);
  return 0;
}

static EVENT WM_command_IDCANCEL(tMSGParam *msg)
{
  return DestroyDialog(msg->hWnd, TRUE);
}

/***************************************************************************/
tMSGEntry ScalingMsgTbl[] = {
  { WM_INITDIALOG , DoC , DoC , WM_initdialog,  LOCK  }, /*a*/
  { WM_COMMAND , DO_SHOW_OUT , DoC , WM_command_DO_SHOW_OUT,  LOCK  }, /*a*/
  { WM_COMMAND , DO_SHOW_MAP , DoC , WM_command_DO_SHOW_MAP,  LOCK  }, /*a*/
  { WM_COMMAND , ID_SCALE , EN_CHANGE , WM_command_ID_SCALE__EN_CHANGE,  LOCK  }, /*a*/
  { WM_COMMAND , ID_MAP_DIST , EN_CHANGE , WM_command_ID_MAP_DIST__EN_CHANGE,  LOCK  }, /*a*/
  { WM_COMMAND , ID_OUT_DIST , EN_CHANGE , WM_command_ID_OUT_DIST__EN_CHANGE,  LOCK  }, /*a*/
  { WM_COMMAND , ID_MAP_UNIT , CBN_SELCHANGE , WM_command_ID_MAP_UNIT__CBN_SELCHANGE,  LOCK  }, /*a*/
  { WM_COMMAND , ID_OUT_UNIT , CBN_SELCHANGE , WM_command_ID_OUT_UNIT__CBN_SELCHANGE,  LOCK  }, /*a*/
  { WM_COMMAND , IDR_THIS , DoC , WM_command_IDR_THIS,  LOCK  }, /*a*/
  { WM_COMMAND , IDR_OTHER , DoC , WM_command_IDR_OTHER,  LOCK  }, /*a*/
  { WM_COMMAND , IDOK , DoC , WM_command_IDOK }, /*a*/
  { WM_COMMAND , IDCANCEL , DoC , WM_command_IDCANCEL }, /*a*/
  { 0 }
};
/***************************************************************************/