/**************/
/* scaling.c  */
/****************************************************************************
    PROGRAM: scaling.c
    PURPOSE: Vaihtaa matkoja
             Huolehditaan scaling-dialogin ksittelyst.
             Vain yhdet staattiset muuttujat => dialogista ei voi
             olla useita yhtaikaisia esiintymi.

****************************************************************************/
#include <windowsx.h>
#include "portable.h"
#include "mdialog.h"
#include "showdoub.h"
#include "tabhand.h"
#include "vaihtaja.h"

/***************************************************************************/
typedef struct {
  char   *unit;
  double mm;
} unit_type;

static unit_type units[] = {
  { "mm",                1.0     },
  { "cm",               10.0     },
  { "dm",              100.0     },
  { "m",              1000.0     },
  { "km",          1000000.0     },
  { "inch",             25.4     },
  { "ft",              304.8     },
  { "yard",            914.4     },
  { "mile",        1609300.0     },
  { "NM",          1853200.0     },
  { "tuuma(s)",         24.742   },
  { "kortteli",        148.45    },
  { "jalka(s)",        296.9     },
  { "kyynr",         593.8     },
  { "syli",           1781.4     },
  { "virsta",      1068800.0     },
  { "peninkulma", 10688000.0     },
  { NULL,                0.0     }
};

#define MM 0
#define KM 4

#define DO_SHOW_OUT 1991
#define DO_SHOW_MAP 1992

#define H(id) GetDlgItem(msg->hWnd,id)
#define SEND_COMMAND(id) SEND_WM_COMMAND(msg->hWnd,id,GetDlgItem(msg->hWnd,id),0)

static int    map_unit     = MM;          /* mm */
static double map_dist     = 10.0;
static int    out_unit     = KM;          /* km */
static double out_dist     = 2.0;
static double scale        = 200000.0;
static int    change_other = 1;

/***************************************************************************/
static EVENT WM_initdialog(tMSGParam *msg)
{
  int i;
  HWND hCBMapU = GetDlgItem(msg->hWnd,ID_MAP_UNIT);
  HWND hCBOutU = GetDlgItem(msg->hWnd,ID_OUT_UNIT);
  for (i=0; units[i].unit; i++) {  /* Tytetn combo-listat */
    (void)ComboBox_InsertString(hCBMapU,i,units[i].unit);
    (void)ComboBox_InsertString(hCBOutU,i,units[i].unit);
  }

  (void)ComboBox_SetCurSel(hCBMapU,map_unit);               /* oletus yks  */
  (void)ComboBox_SetCurSel(hCBOutU,out_unit);               /* oletus yks  */
  (void)Button_SetCheck(H(IDR_OTHER),(change_other=1));     /* oletus muut */

  ShowDouble(msg->hWnd,ID_SCALE,scale);
  ShowDouble(msg->hWnd,ID_MAP_DIST,map_dist);
  ShowDouble(msg->hWnd,ID_OUT_DIST,out_dist);

  return TRUE;
}

static EVENT WM_command_DO_SHOW_OUT(tMSGParam *msg)
{
  out_dist = map_dist*units[map_unit].mm*scale/units[out_unit].mm;
  return ShowDouble(msg->hWnd,ID_OUT_DIST,out_dist);
}

static EVENT WM_command_DO_SHOW_MAP(tMSGParam *msg)
{
  map_dist = out_dist/(units[map_unit].mm*scale/units[out_unit].mm);
  return ShowDouble(msg->hWnd,ID_MAP_DIST,map_dist);
}

static EVENT WM_command_ID_SCALE__EN_CHANGE(tMSGParam *msg)
{
  scale = GetDouble(msg->hWnd,msg->wParam,1);
  if ( scale <= 0 ) scale = 1;
  return SEND_COMMAND(DO_SHOW_OUT);
}

static EVENT WM_command_ID_MAP_DIST__EN_CHANGE(tMSGParam *msg)
{
  map_dist = GetDouble(msg->hWnd,msg->wParam,0);
  return SEND_COMMAND(DO_SHOW_OUT);
}

static EVENT WM_command_ID_OUT_DIST__EN_CHANGE(tMSGParam *msg)
{
  out_dist = GetDouble(msg->hWnd,msg->wParam,0);
  return SEND_COMMAND(DO_SHOW_MAP);
}

static EVENT WM_command_ID_MAP_UNIT__CBN_SELCHANGE(tMSGParam *msg)
{
  map_unit = ComboBox_GetCurSel(H(ID_MAP_UNIT));
  return SEND_COMMAND(change_other ? DO_SHOW_OUT : DO_SHOW_MAP);
}

static EVENT WM_command_ID_OUT_UNIT__CBN_SELCHANGE(tMSGParam *msg)
{
  out_unit = ComboBox_GetCurSel(H(ID_OUT_UNIT));
  return SEND_COMMAND(change_other ? DO_SHOW_MAP : DO_SHOW_OUT);
}

static EVENT WM_command_IDR_THIS(tMSGParam *msg)
{
  change_other = 0;
  return msg->extra;
}

static EVENT WM_command_IDR_OTHER(tMSGParam *msg)
{
  change_other = 1;
  return msg->extra;
}

static EVENT WM_command_IDOK(tMSGParam *msg)
{
  SetFocus(H(ID_MAP_DIST)); Edit_SetSel(H(ID_MAP_DIST),0,-1);
  return 0;
}

static EVENT WM_command_IDCANCEL(tMSGParam *msg)
{
  return DestroyDialog(msg->hWnd, TRUE);
}

/***************************************************************************/
tMSGEntry ScalingMsgTbl[] = {
  { WM_INITDIALOG , DoC , DoC, WM_initdialog }, /*a*/
  { WM_COMMAND , DO_SHOW_OUT , DoC, WM_command_DO_SHOW_OUT }, /*a*/
  { WM_COMMAND , DO_SHOW_MAP , DoC, WM_command_DO_SHOW_MAP }, /*a*/
  { WM_COMMAND , ID_SCALE , EN_CHANGE, WM_command_ID_SCALE__EN_CHANGE }, /*a*/
  { WM_COMMAND , ID_MAP_DIST , EN_CHANGE, WM_command_ID_MAP_DIST__EN_CHANGE }, /*a*/
  { WM_COMMAND , ID_OUT_DIST , EN_CHANGE, WM_command_ID_OUT_DIST__EN_CHANGE }, /*a*/
  { WM_COMMAND , ID_MAP_UNIT , CBN_SELCHANGE, WM_command_ID_MAP_UNIT__CBN_SELCHANGE }, /*a*/
  { WM_COMMAND , ID_OUT_UNIT , CBN_SELCHANGE, WM_command_ID_OUT_UNIT__CBN_SELCHANGE }, /*a*/
  { WM_COMMAND , IDR_THIS , DoC, WM_command_IDR_THIS }, /*a*/
  { WM_COMMAND , IDR_OTHER , DoC, WM_command_IDR_OTHER }, /*a*/
  { WM_COMMAND , IDOK , DoC, WM_command_IDOK }, /*a*/
  { WM_COMMAND , IDCANCEL , DoC, WM_command_IDCANCEL }, /*a*/
  { 0 }
};
/***************************************************************************/